{% extends "security/base.html" %}
{% from "security/_macros.html" import render_field_with_errors, render_field, header %}

{% block content %}
{% include "security/_messages.html" %}
{{ header(_('Send password reset instructions')) }}
<form action="{{ url_for_security('forgot_password') }}" method="POST" name="forgot_password_form">
  {{ forgot_password_form.hidden_tag() }}
  {{ render_field_with_errors(forgot_password_form.email) }}
  {{ render_field(forgot_password_form.submit, class_="button is-link") }}
</form>
{% endblock %}
